(defface column-marker-1 '((t (:background "gray"))) "" :group 'faces)

(defvar column-marker-vars ())
(make-variable-buffer-local 'column-marker-vars)

(defvar column-marker-1 ())
(make-variable-buffer-local 'column-marker-1)
(defun column-marker-1 (arg)
  (interactive "P")
  (unless (memq 'column-marker-1 column-marker-vars)
    (push 'column-marker-1 column-marker-vars))
  (cond ((null arg)
         (column-marker-internal
          'column-marker-1 (1+ (current-column)) 'column-marker-1))
        ((consp arg)
         (if (= 4 (car arg))
             (column-marker-internal 'column-marker-1 nil)
           (dolist (var column-marker-vars)
             (column-marker-internal var nil))))
        ((and (integerp arg) (>= arg 0))
         (column-marker-internal 'column-marker-1
                                 (1+ (prefix-numeric-value arg))
                                 'column-marker-1))
        (t
         (dolist (var column-marker-vars)
           (column-marker-internal var nil)))))

(defun column-marker-find (col)
  "Defines a function to locate a character in column COL.
Returns the function symbol, named `column-marker-move-to-COL'."
  (let ((fn-symb  (intern (format "column-marker-move-to-%d" col))))
    (fset `,fn-symb
          `(lambda (end)
             (let ((start (point)))
               (when (> end (point-max)) (setq end (point-max)))

               (unless (< (current-column) ,col) (forward-line 1))
               (when (< (current-column) ,col) (move-to-column ,col))
               (while (and (< (current-column) ,col) (< (point) end)
                           (= 0 (+ (forward-line 1) (current-column))))
                 (move-to-column ,col))
               (if (and (= ,col (current-column))
                        (<= (point) end) (> (point) start))
                   (progn (set-match-data (list (1- (point)) (point))) t)
                 (goto-char start)
                 nil))))
    fn-symb))

(defun column-marker-internal (sym col &optional face)
  "SYM is the symbol for holding the column marker context.
COL is the column in which a marker should be set.
Supplying nil or 0 for COL turns off the marker.
FACE is the face to use.  If nil, then face `column-marker-1' is used."
  (setq face (or face 'column-marker-1))
  (when (symbol-value sym)
    (font-lock-remove-keywords nil (symbol-value sym))
    (set sym nil))
  (when (or (listp col) (< col 0)) (setq col nil))
  (when col
    (set sym `((,(column-marker-find col) (0 ,face prepend t))))
    (font-lock-add-keywords nil (symbol-value sym) t))
  (font-lock-fontify-buffer))


;; --------------------------------------------------------
;; These are the messages in *Messages*, with this sequence repeated a
;; zillion times. Note that the messages do *not* appear as soon as I
;; call `M-x column-marker-1' (with point in column 21).  They appear
;; (I think) as soon as I do something that forces redisplay -
;; e.g. select a window showing *Messages*.
;;
;; Invalid face reference: prepend
;; Invalid face reference: 0
;; Invalid face reference: column-marker-move-to-22
;; Invalid face reference: t

;;-----------------------------------------------
;; This is (symbol-function 'column-marker-1):
;; (lambda
;;     (arg)
;;   (interactive "P")
;;   (unless
;;       (memq 'column-marker-1 column-marker-vars)
;;     (push 'column-marker-1 column-marker-vars))
;;   (cond
;;     ((null arg)
;;      (column-marker-internal 'column-marker-1
;;                              (1+
;;                               (current-column))
;;                              'column-marker-1))
;;     ((consp arg)
;;      (if
;;       (= 4
;;          (car arg))
;;       (column-marker-internal 'column-marker-1 nil)
;;        (dolist
;;            (var column-marker-vars)
;;          (column-marker-internal var nil))))
;;     ((and
;;       (integerp arg)
;;       (>= arg 0))
;;      (column-marker-internal 'column-marker-1
;;                              (1+
;;                               (prefix-numeric-value arg))
;;                              'column-marker-1))
;;     (t
;;      (dolist
;;          (var column-marker-vars)
;;        (column-marker-internal var nil)))))

;;-----------------------------------------------
;; This the *value* of `column-marker-1', when highlighted:
;; ((column-marker-move-to-22 (0 column-marker-1 prepend t)))

;;-----------------------------------------------
;; This is (symbol-function 'column-marker-move-to-22) when highlighted:
;; (lambda (end)
;;   (let ((start (point)))
;;     (when (> end (point-max))
;;       (setq end (point-max)))
;;     (unless (< (current-column) 22) (forward-line 1))
;;     (when (< (current-column) 22) (move-to-column 22))
;;     (while (and (< (current-column) 22)
;;                 (< (point) end)
;;                 (= 0 (+ (forward-line 1)(current-column))))
;;       (move-to-column 22))
;;     (if (and (= 22 (current-column))
;; 	     (<= (point) end)
;; 	     (> (point) start))
;;         (progn
;;           (set-match-data
;;            (list (1- (point)) (point)))
;;           t)
;;       (goto-char start)
;;       nil)))

;;-----------------------------------------------
;; This is the part of `font-lock-keywords' that is relevant:
;; (column-marker-move-to-22 (0 column-marker-1 prepend t))

;;-----------------------------------------------
;; This is the full value of `font-lock-keywords':
;; (t
;;  (("(\\(def\\(\\(advice\\|alias\\|generic\\|macro\\*?\\|method\\|setf\\|subst\\*?\\|un\\*?\\|ine-\\(condition\\|\\(?:derived\\|\\(?:global\\(?:ized\\)?-\\)?minor\\|generic\\)-mode\\|method-combination\\|setf-expander\\|skeleton\\|widget\\|function\\|\\(compiler\\|modify\\|symbol\\)-macro\\)\\)\\|\\(const\\(ant\\)?\\|custom\\|varalias\\|face\\|parameter\\|var\\)\\|\\(class\\|group\\|theme\\|package\\|struct\\|type\\)\\)\\)\\>[ 	'(]*\\(setf[ 	]+\\sw+)\\|\\sw+\\)?"
;;    (1 font-lock-keyword-face)
;;    (9
;;     (cond
;;      ((match-beginning 3)
;;       font-lock-function-name-face)
;;      ((match-beginning 6)
;;       font-lock-variable-name-face)
;;      (t font-lock-type-face))
;;     nil t))
;;   ("^;;;###\\(autoload\\)" 1 font-lock-warning-face prepend)
;;   ("\\[\\(\\^\\)" 1 font-lock-negation-char-face prepend)
;;   ("(\\(cond\\(?:ition-case\\)?\\|eval-\\(?:a\\(?:fter-load\\|nd-compile\\|t-startup\\)\\|next-after-load\\|when\\(?:-compile\\)?\\)\\|i\\(?:f\\|nline\\)\\|l\\(?:ambda\\|et\\*?\\)\\|prog[*12nv]?\\|save-\\(?:current-buffer\\|excursion\\|match-data\\|restriction\\|selected-window\\|window-excursion\\)\\|track-mouse\\|unwind-protect\\|w\\(?:hile\\(?:-no-input\\)?\\|ith-\\(?:c\\(?:a\\(?:\\(?:se\\|tegory\\)-table\\)\\|urrent-buffer\\)\\|electric-help\\|local-quit\\|no-warnings\\|output-to-\\(?:string\\|temp-buffer\\)\\|s\\(?:elected-window\\|yntax-table\\)\\|t\\(?:emp-\\(?:buffer\\|\\(?:fil\\|messag\\)e\\)\\|imeout\\(?:-handler\\)?\\)\\)\\)\\)\\>" . 1)
;;   ("(\\(b\\(?:\\(?:loc\\|rea\\)k\\)\\|c\\(?:ase\\|case\\|ompiler-let\\|typecase\\)\\|d\\(?:e\\(?:cla\\(?:im\\|re\\)\\|structuring-bind\\)\\|o\\(?:\\*\\|list\\|times\\)?\\)\\|e\\(?:\\(?:type\\)?case\\)\\|flet\\|go\\|handler-\\(?:bind\\|case\\)\\|i\\(?:gnore-errors\\|n-package\\)\\|l\\(?:abels\\|exical-let\\*?\\|o\\(?:cally\\|op\\)\\)\\|m\\(?:acrolet\\|ultiple-value-\\(?:bind\\|prog1\\)\\)\\|proclaim\\|re\\(?:start-\\(?:bind\\|case\\)\\|turn\\(?:-from\\)?\\)\\|symbol-macrolet\\|t\\(?:agbody\\|\\(?:h\\|ypecas\\)e\\)\\|unless\\|w\\(?:hen\\|ith-\\(?:accessors\\|co\\(?:mpilation-unit\\|ndition-restarts\\)\\|hash-table-iterator\\|input-from-string\\|o\\(?:pen-\\(?:file\\|stream\\)\\|utput-to-string\\)\\|package-iterator\\|s\\(?:imple-restart\\|lots\\|tandard-io-syntax\\)\\)\\)\\)\\>" . 1)
;;   ("(\\(catch\\|throw\\|featurep\\|provide\\|require\\)\\>[ 	']*\\(\\sw+\\)?"
;;    (1 font-lock-keyword-face)
;;    (2 font-lock-constant-face nil t))
;;   ("(\\(abort\\|assert\\|warn\\|check-type\\|cerror\\|error\\|signal\\)\\>" 1 font-lock-warning-face)
;;   ("\\\\\\\\\\[\\(\\sw+\\)\\]" 1 font-lock-constant-face prepend)
;;   ("`\\(\\sw\\sw+\\)'" 1 font-lock-constant-face prepend)
;;   ("\\<:\\sw+\\>" 0 font-lock-builtin-face)
;;   ("\\<\\&\\sw+\\>" . font-lock-type-face)
;;   ((lambda
;;      (bound)
;;      (catch 'found
;;        (while
;; 	   (re-search-forward "\\(\\\\\\\\\\)\\(?:\\(\\\\\\\\\\)\\|\\((\\(?:\\?:\\)?\\|[|)]\\)\\)" bound t)
;; 	 (unless
;; 	     (match-beginning 2)
;; 	   (let
;; 	       ((face
;; 		 (get-text-property
;; 		  (1-
;; 		   (point))
;; 		  'face)))
;; 	     (when
;; 		 (or
;; 		  (and
;; 		   (listp face)
;; 		   (memq 'font-lock-string-face face))
;; 		  (eq 'font-lock-string-face face))
;; 	       (throw 'found t)))))))
;;    (1 'font-lock-regexp-grouping-backslash prepend)
;;    (3 'font-lock-regexp-grouping-construct prepend))
;;   (column-marker-move-to-22
;;    (0 column-marker-1 prepend t)))
;;  ("(\\(def\\(\\(advice\\|alias\\|generic\\|macro\\*?\\|method\\|setf\\|subst\\*?\\|un\\*?\\|ine-\\(condition\\|\\(?:derived\\|\\(?:global\\(?:ized\\)?-\\)?minor\\|generic\\)-mode\\|method-combination\\|setf-expander\\|skeleton\\|widget\\|function\\|\\(compiler\\|modify\\|symbol\\)-macro\\)\\)\\|\\(const\\(ant\\)?\\|custom\\|varalias\\|face\\|parameter\\|var\\)\\|\\(class\\|group\\|theme\\|package\\|struct\\|type\\)\\)\\)\\>[ 	'(]*\\(setf[ 	]+\\sw+)\\|\\sw+\\)?"
;;   (1 font-lock-keyword-face)
;;   (9
;;    (cond
;;     ((match-beginning 3)
;;      font-lock-function-name-face)
;;     ((match-beginning 6)
;;      font-lock-variable-name-face)
;;     (t font-lock-type-face))
;;    nil t))
;;  ("^;;;###\\(autoload\\)"
;;   (1 font-lock-warning-face prepend))
;;  ("\\[\\(\\^\\)"
;;   (1 font-lock-negation-char-face prepend))
;;  ("(\\(cond\\(?:ition-case\\)?\\|eval-\\(?:a\\(?:fter-load\\|nd-compile\\|t-startup\\)\\|next-after-load\\|when\\(?:-compile\\)?\\)\\|i\\(?:f\\|nline\\)\\|l\\(?:ambda\\|et\\*?\\)\\|prog[*12nv]?\\|save-\\(?:current-buffer\\|excursion\\|match-data\\|restriction\\|selected-window\\|window-excursion\\)\\|track-mouse\\|unwind-protect\\|w\\(?:hile\\(?:-no-input\\)?\\|ith-\\(?:c\\(?:a\\(?:\\(?:se\\|tegory\\)-table\\)\\|urrent-buffer\\)\\|electric-help\\|local-quit\\|no-warnings\\|output-to-\\(?:string\\|temp-buffer\\)\\|s\\(?:elected-window\\|yntax-table\\)\\|t\\(?:emp-\\(?:buffer\\|\\(?:fil\\|messag\\)e\\)\\|imeout\\(?:-handler\\)?\\)\\)\\)\\)\\>"
;;   (1 font-lock-keyword-face))
;;  ("(\\(b\\(?:\\(?:loc\\|rea\\)k\\)\\|c\\(?:ase\\|case\\|ompiler-let\\|typecase\\)\\|d\\(?:e\\(?:cla\\(?:im\\|re\\)\\|structuring-bind\\)\\|o\\(?:\\*\\|list\\|times\\)?\\)\\|e\\(?:\\(?:type\\)?case\\)\\|flet\\|go\\|handler-\\(?:bind\\|case\\)\\|i\\(?:gnore-errors\\|n-package\\)\\|l\\(?:abels\\|exical-let\\*?\\|o\\(?:cally\\|op\\)\\)\\|m\\(?:acrolet\\|ultiple-value-\\(?:bind\\|prog1\\)\\)\\|proclaim\\|re\\(?:start-\\(?:bind\\|case\\)\\|turn\\(?:-from\\)?\\)\\|symbol-macrolet\\|t\\(?:agbody\\|\\(?:h\\|ypecas\\)e\\)\\|unless\\|w\\(?:hen\\|ith-\\(?:accessors\\|co\\(?:mpilation-unit\\|ndition-restarts\\)\\|hash-table-iterator\\|input-from-string\\|o\\(?:pen-\\(?:file\\|stream\\)\\|utput-to-string\\)\\|package-iterator\\|s\\(?:imple-restart\\|lots\\|tandard-io-syntax\\)\\)\\)\\)\\>"
;;   (1 font-lock-keyword-face))
;;  ("(\\(catch\\|throw\\|featurep\\|provide\\|require\\)\\>[ 	']*\\(\\sw+\\)?"
;;   (1 font-lock-keyword-face)
;;   (2 font-lock-constant-face nil t))
;;  ("(\\(abort\\|assert\\|warn\\|check-type\\|cerror\\|error\\|signal\\)\\>"
;;   (1 font-lock-warning-face))
;;  ("\\\\\\\\\\[\\(\\sw+\\)\\]"
;;   (1 font-lock-constant-face prepend))
;;  ("`\\(\\sw\\sw+\\)'"
;;   (1 font-lock-constant-face prepend))
;;  ("\\<:\\sw+\\>"
;;   (0 font-lock-builtin-face))
;;  ("\\<\\&\\sw+\\>"
;;   (0 font-lock-type-face))
;;  ((lambda
;;     (bound)
;;     (catch 'found
;;       (while
;; 	  (re-search-forward "\\(\\\\\\\\\\)\\(?:\\(\\\\\\\\\\)\\|\\((\\(?:\\?:\\)?\\|[|)]\\)\\)" bound t)
;; 	(unless
;; 	    (match-beginning 2)
;; 	  (let
;; 	      ((face
;; 		(get-text-property
;; 		 (1-
;; 		  (point))
;; 		 'face)))
;; 	    (when
;; 		(or
;; 		 (and
;; 		  (listp face)
;; 		  (memq 'font-lock-string-face face))
;; 		 (eq 'font-lock-string-face face))
;; 	      (throw 'found t)))))))
;;   (1 'font-lock-regexp-grouping-backslash prepend)
;;   (3 'font-lock-regexp-grouping-construct prepend))
;;  (column-marker-move-to-22
;;   (0 column-marker-1 prepend t))
;;  ("^\\s("
;;   (0
;;    (if
;;        (memq
;; 	(get-text-property
;; 	 (match-beginning 0)
;; 	 'face)
;; 	'(font-lock-string-face font-lock-doc-face font-lock-comment-face))
;;        (list 'face font-lock-warning-face 'help-echo "Looks like a toplevel defun: escape the parenthesis"))
;;    prepend)))

;;---------------------------------------------------------
;; For reference, this is a macro used in the original code.  I simply
;; inlined a call to it with arg `column-marker-1', above, to simplify things.

;; ;; (defmacro column-marker-create (var &optional face)
;;   "Define a column marker named VAR.
;; FACE is the face to use.  If nil, then face `column-marker-1' is used."
;;   (setq face (or face 'column-marker-1))
;;   `(progn
;;      (defvar ,var () "Buffer local. Used internally to store column marker spec.")
;;      (make-variable-buffer-local ',var)
;;      (defun ,var (arg)
;;        (interactive "P")
;;        (unless (memq ',var column-marker-vars) (push ',var column-marker-vars))
;;        (cond ((null arg)
;;               (column-marker-internal ',var (1+ (current-column)) ,face))
;;              ((consp arg)
;;               (if (= 4 (car arg))
;;                   (column-marker-internal ',var nil)
;;                 (dolist (var column-marker-vars)
;;                   (column-marker-internal var nil))))
;;              ((and (integerp arg) (>= arg 0))
;;               (column-marker-internal ',var (1+ (prefix-numeric-value arg)) ,face))
;;              (t
;;               (dolist (var column-marker-vars)
;;                 (column-marker-internal var nil)))))))
;;
;;(column-marker-create column-marker-1)


